/*
 * $Header: D:/Projects/Repository/GaugeAPI/edu/cmu/cs/able/gauges/RMI/RMITimeGaugeMgr.java,v 1.5 2002/05/24 18:11:59 schmerl Exp $
 * $Log: RMITimeGaugeMgr.java,v $
 * Revision 1.5  2002/05/24 18:11:59  schmerl
 * Removed delegation from Gauges (now in RMI Gauge).
 * Also, removed classpath from gauge type.
 * Gauge managers now respond to gauges of a particular type,
 * and have hardwired internally the particular class to start.
 *
 * Revision 1.4  2002/01/31 16:39:11  schmerl
 * Added CVS header information
 *
*/
package edu.cmu.cs.able.gauges.RMI;
import java.rmi.*;
import java.util.*;

import java.net.*;
import java.io.*;

import edu.cmu.cs.able.gaugeInfrastructure.*;
import edu.cmu.cs.able.gaugeInfrastructure.util.*;
import edu.cmu.cs.able.gaugeInfrastructure.RMI.*;

public class RMITimeGaugeMgr extends RMIGaugeMgr {
    Hashtable gaugeProcessTable = new Hashtable(); 

    private String implementationClass = "edu.cmu.cs.able.gauges.RMI.RMITimeGauge";

    /** Creates new RMITimeGaugeMgr */
    public RMITimeGaugeMgr(String bushost) {
        super(bushost);
        try {
            this.busHost = bushost;
            gaugeBus = new RMIGaugeReportingBus(bushost);
            gaugeMgrType = "TimeGauge";
            gaugeMgrUID = Global.generateUID();
            String host = Initialization.startGaugeMgrService(this);
            gaugeBus.registerGaugeMgr(gaugeMgrUID, gaugeMgrType, host);
            /*GaugeID gaugeID = new GaugeID();
            gaugeID.gaugeName = "RMITimeGaugeName";
            gaugeID.gaugeType = gaugeMgrType;
            gaugeID.modelName = "RMITimeGaugeModelName";
            gaugeID.modelType = "RMITimeGaugeModelType";
            gaugeID.uid = "";
            createGauge(gaugeID, null, null);*/
        }
        catch (Exception e) {}
    }
    /** Creates a new gauge as a separate process.
     * @param gauge The ID of the gauge to create.
     * @param setupParams The setup parameters to pass to the new gauge.
     * @param mappings The mappings to pass to the new gauge.
     * @return A gauge control that has the new ID and setup parameters.
     */    
    public GaugeControl createGauge(GaugeID gauge, StringPairVector setupParams, StringPairVector mappings) {
        String setup = "";
        for (int i = 0; i < setupParams.size()-1; i++) {
            setup = setup+setupParams.nameAt(i)+"="+setupParams.valueAt(i)+" ";
        }
        if ( setupParams.size() > 0) {
            setup = setup+setupParams.nameAt( setupParams.size()-1)+"="+setupParams.valueAt( setupParams.size()-1);
        }
        
        String uid = Global.generateUniqueID();
        gauge.uid = uid;    
        String cmd = "java  "+implementationClass+" -gn "+gauge.gaugeName+" -gt "+gauge.gaugeType+" -mn "+gauge.modelName+" -mt "+gauge.modelType+" -uid "+uid+" -setup "+setup+" -rmib "+busHost; 
        
        System.out.println(cmd);
        Process gaugeProcess = Global.exec(cmd);
        String host = null;
        int timeout = 20;
        RMIGaugeHandle gh = null;
        GaugeID gaugeID = null;
        try {
            host = gaugeBus.getGaugeHost(gauge);
            while (host == null && timeout > 0) {
                Thread.sleep(1000);
                timeout--;
                host = gaugeBus.getGaugeHost(gauge);
            }
            gh = new RMIGaugeHandle(host);
            gaugeID = gh.getGaugeID();
        }
        catch (Exception e) {}
        
        System.err.println(host);
        String gid = gauge.toString();
        gaugeProcessTable.put(gid, gaugeProcess);
        System.out.println(gaugeProcessTable.toString());
        System.out.println(gaugeID.toString());
        return gh;
    }

    /** Deletes the gauge process associated with the gauge ID.
     * @param gauge The gauge to be deleted.
     * @return Whether the process was destroyed successfully.
     */    
    public boolean deleteGauge(GaugeControl gauge) {
        System.out.println(gaugeProcessTable.toString());
        Process gaugeProcess = (Process )gaugeProcessTable.get (gauge.getGaugeID ().toString());
        System.out.println("try to kill the gauge process"+gaugeProcess.toString());
        System.out.println(gauge.getGaugeID ().toString());
        if (gaugeProcess != null) {
            // Delete the gauge process
            try {
                System.out.println("try to kill the gauge process"+gaugeProcess.toString());
                gaugeProcess.destroy ();
                System.out.println("gauge process killed");
            }
            catch (Throwable t) {
                System.err.println ("Could not destroy gauge");
                t.printStackTrace ();
                return false;
            }
        }
        return true;
    }

    /** Returns true if the gauge type is edu.cmu.cs.able.gauges.RMI.RMITimeGauge
     * @param gaugeType
     * @return
     */    
    public boolean managesType(String gaugeType) {
        return gaugeType.equals ("edu.cmu.cs.able.gauges.RMI.RMITimeGauge");
    }
    
    /** Returns the parameters that can be used to configure the gauge,
     * as well as the values reported by the gauge, for a particular
     * gauge type.
     * @param gaugeType The type of the gauge about which to get information.
     * @param configParamsMeta Will fill this vector with [("frequency", "int"), ("report", "boolean")]
     * @param valuesMeta Will fill this vector with [("date", "string")]
     * @return
     */    
    public boolean queryMetaInfo(String gaugeType, StringPairVector configParamsMeta, StringPairVector valuesMeta) {
        System.out.println("meta");
        if (managesType (gaugeType)) {
            configParamsMeta.addElement ("frequency", "int");
            configParamsMeta.addElement ("report", "boolean");
            valuesMeta.addElement ("date", "string");
            System.out.println(configParamsMeta.toString());
            return true;
        }
        else {
            return false;
        }
    }
    
    
    public static void main(String args[]) {
        if (args.length < 1) {
            System.err.println("Eventbus host missing");
            return;
        }
        
        RMITimeGaugeMgr theTimeGaugeMgr = new RMITimeGaugeMgr(args[0]);
    }
}

