/*
 * SienaLatencyGauge.java
 *
 * Created on November 19, 2001, 4:43 PM
 */

package edu.cmu.cs.able.gauges.Siena.Demo;

import java.util.Enumeration;
import edu.cmu.cs.able.gaugeInfrastructure.Siena.*;
import edu.cmu.cs.able.gaugeInfrastructure.util.*;
import edu.cmu.cs.able.gaugeInfrastructure.Events.*;
import edu.cmu.cs.able.probebus.*;
import edu.cmu.cs.able.probebus.siena.*;
import edu.cmu.cs.able.probebus.events.*;
import edu.cmu.cs.able.probebus.siena.events.*;

public class ProtocolGauge extends edu.cmu.cs.able.gaugeInfrastructure.Siena.SienaGauge {
    
    public String gaugeType = "edu.cmu.cs.able.gauges.Siena.Demo.ProtocolGauge";
    
    private static IProbeBus probeBus = null;

    private ProtocolGaugeImpl impl = null;
    /** Creates new SienaProtocolGauge     
     * @param gaugeID The ID of the new gauge.
     * @param creatorID The ID of the gauge manager that created the gauge.
     */
    public ProtocolGauge (GaugeID gaugeID, String creatorID, StringPairVector setupParams, StringPairVector mappings) {
        super (gaugeID, creatorID, setupParams, mappings);
        
        impl = new ProtocolGaugeImpl (gaugeID, setupParams, mappings, gaugeBus, probeBus);

        //int retries = (int)(30 / (gaugeID.retryCnt.intValue()+1));
        int retries = 10;
        int i = 0;
        for (; i < retries && !impl.consistentlyCreated (); i++) {
            Global.debug ("ProtocolGauge sleeping..." + i);
            try {
                Thread.sleep (1000);
            } catch (InterruptedException e) {
                if (impl.consistentlyCreated()) {  // probe event caused interrupt!?
                    break;
                }// otherwise, just keep retrying until max retries
            }
        }
        
        if (i >= retries) {
            Global.debug ("Timed out deploying probes.");
        }

        finalizeCreation (creatorID, impl.consistentlyCreated ());
    }
    
    /** This method is used to configure the gauge.
      */    
    public boolean configure(StringPairVector configParams) {
        return impl.configure (configParams);
    }
    
    /** Returns the current date.
     * @param values The vector into which to place the date.
     * @return Whether the gauge could report it's current value.
     */    
    public boolean queryAllValues (GaugeValueVector values) {
        return impl.queryAllValues (values);
    }
    
    /** Returns the state of the gauge.
     */    
    public boolean queryState(StringPairVector setupParams, StringPairVector configParams, StringPairVector mappings) {
        return impl.queryState (setupParams, configParams, mappings);
    }
    
    /** The method called to query the value of the gauge. This
     * provides a direct query for the gauge, as opposed to the
     * gauge reporting to value on the bus.
     * @param valueName The name of the value to query.
     * @return The corresponding value of the gauge. (Note: <I>it is assumed
     * that the querier know the property associated with this
     * value</I>).
     */    
    public String queryValue(String valueName) {
        return impl.queryValue (valueName);
    }
    

    public void processDelete (String gaugeMgrID) {
        impl.protocolProbe.manageRemoval ();
        finalizeDeletion (gaugeMgrID);
        System.out.println("Gauge: " + getGaugeID ().gaugeName + " exiting");
        impl.protocolProbe.shutdown ();
    }
    /** A command-line interface to start the gauge.
     * @param args -gt <i>gauge type</i><br>
     * -gn <i>gauge name</i><br>
     * -mt <i>model type</i><br>
     * -mn <i>model name</i><br>
     * [-setup <i>name=value</i>]<br>
     * [-mappings <i>name=property</i>]<br>
     * -creator <i>creator ID</i><br>
     * -senp <i>siena port</i><br>
     * [-debug]
     */    
    public static void main (String [] args) {

        
        String creatorID = null;
        String gaugeName = null;
        String gaugeType = null;
        String modelName = null;
        String modelType = null;
        StringPairVector mappings = new StringPairVector ();
        StringPairVector setupParams = new StringPairVector ();
        Integer retries = null;
        boolean gotMappings = false;
        boolean gotSetup = false;
        // Process args
        boolean ok = true;
        for (int i = 0; i < args.length && ok; i++) {
            String tag = args [i];
            if (args [i].equals ("-creator")) {
                creatorID = args [++i];
            }
            else if (args [i].equals ("-gn")) {
                gaugeName = args [++i];
            }
            else if (args [i].equals ("-gt")) {
                gaugeType = args [++i];
            }
            else if (args [i].equals ("-mn")) {
                modelName = args [++i];
            }
            else if (args [i].equals ("-mt")) {
                modelType = args [++i];
            }
            else if (args [i].equals ("-setup")) {
                gotSetup = true;
                i++;
                while (i < args.length && args [i].charAt (0) != '-') {
                    String setup = args [i];
                    String name = setup.substring (0, setup.indexOf ('='));
                    String value = setup.substring (setup.indexOf ('=') + 1);
                    setupParams.addElement (name, value);
                    i++;
                } 
                i--;
            }
            else if (args [i].equals ("-mappings")) {
                gotMappings = true;
                i++;
                while (i < args.length && args [i].charAt (0) != '-') {
                    String mapping = args [i];
                    String value = mapping.substring (0, mapping.indexOf ('='));
                    String property = mapping.substring (mapping.indexOf ('=') + 1);
                    mappings.addElement (value, property);
                    i++;
                }
                i--;
            }
            else if (args [i].equals ("-retries")) {
                retries = Integer.valueOf(args [++i]);
            }
            else if (args [i].equals ("-senp")) {
                String senp = args [++i];
                edu.cmu.cs.able.gaugeInfrastructure.Siena.Initialization.initSiena (senp, Initialization.GAUGE);
 //               probeBus = new SienaProbeBus ("senp://snowgum.wv.cs.cmu.edu:7130");
            }
            else if (args [i].equals ("-pb")) {
                probeBus = new SienaProbeBus (args [++i]);
            }
            else if (args [i].equals ("-debug")) {
                edu.cmu.cs.able.gaugeInfrastructure.util.Global.debugFlag = true;
            }
            else {
                ok = false;
            }            
        }
        
        if (!ok) {
            printUsageMessage ();
            System.exit (0);
        }
        
        GaugeID gid = new GaugeID ();
        gid.modelName = modelName;
        gid.modelType = modelType;
        gid.gaugeName = gaugeName;
        gid.gaugeType = gaugeType;
        //gid.retryCnt  = retries;
       
        ProtocolGauge gauge = new ProtocolGauge (gid, creatorID, setupParams, mappings);
    }
    
    private static void printUsageMessage () {
        System.out.println ("Usage: ProtocolGauge -creator <creatorID> -mt <modelType> -mn <modelName> -gt <gaugeType> -gn <gaugeName> -setup <name=value> -mappings <value=property> -senp <sienaport>");
        
    }
    
}
