/*
 * LatencyGaugeImpl.java
 *
 * Created on February 20, 2002, 2:22 PM
 */

package edu.cmu.cs.able.gauges.Siena.Demo;
import java.util.*;
import edu.cmu.cs.able.gaugeInfrastructure.Siena.*;
import edu.cmu.cs.able.gaugeInfrastructure.*;
import edu.cmu.cs.able.gaugeInfrastructure.util.*;
import edu.cmu.cs.able.gaugeInfrastructure.Events.*;
import edu.cmu.cs.able.probebus.*;
import edu.cmu.cs.able.probebus.siena.*;
import edu.cmu.cs.able.probebus.events.*;
import edu.cmu.cs.able.probebus.siena.events.*;
import StateMachine.*;


/** Implementation of the ProtocolGauge, conforming to gauge type:<br>
 * ProtocolGauge = [<br>
 * &nbsp;&nbsp;type = edu.cmu.cs.able.gauges.Siena.Demo.ProtocolGauge<br>
 * &nbsp;&nbsp;setup = [where : string<br>
 * 			ProtocolStateMachineFile : String<br>
 *			ReportOnViolation : boolean<br>
 *			EventFilter : string]<br>
 * &nbsp;&nbsp;config = [ReportOnViolation : boolean]<br>
 * &nbsp;&nbsp;values = [violated  : boolean <br>
 *			 LastEvent : string <br>
 *			 EventHistory : string<br>
 *			 ErrorType : integer]<br>
 * @author Bradley Schmerl
 * @version 1.0
 */
public class ProtocolGaugeImpl extends edu.cmu.cs.able.gaugeInfrastructure.GaugeImpl implements IProbeConsumer {
    
    private SMTracer tracer; 
    
    protected boolean curState = true;
    protected String lastEvent = "";
    protected int errorType;
    protected String smFile = null;
    protected boolean reportOnViolation = false;
    protected String where = null;
    protected String eventFilter = null;
    
    Hashtable mapPG = new Hashtable();
    
    private static IProbeBus probeBus = null;
    private final String CONFIG_NAME = "ProtocolConfigurationModule";
    private final String TARGET_CLASSES = "MainFilter";
    public KXProbeHelper protocolProbe;
    private IProbeEventFactory probeEventFactory = null;    
    Boolean probesActivated = new Boolean (false);
    /** Creates new SienaProtocolGauge     
     * @param gaugeID The ID of the new gauge.
     * @param creatorID The ID of the gauge manager that created the gauge.
     * @param setupParams The parameters that the gauge uses to set itself up. 
     */
    public ProtocolGaugeImpl (GaugeID gid, StringPairVector setup, StringPairVector mappings, GaugeReportingBus gaugeBus, IProbeBus probeBus) {
        super (gid, setup, mappings, gaugeBus);
        // moved this earlier in the construction to ensure its effect when GaugeImpl has started reporting
        reportingFrequency = 3000;
        this.probeBus = probeBus;
        
        //setup event mapping from probe to gauge.
        mapPG.put(PipeFilterMessage.WRITE_PIPE, "write");
        mapPG.put(PipeFilterMessage.CLOSE_PIPE, "close");
        
        
        where = setup.getValue ("where");
        smFile = setup.getValue ("ProtocolStateMachineFile");
        reportOnViolation = Boolean.getBoolean(setup.getValue ("ReportOnViolation"));
        eventFilter = setup.getValue ("EventFilter");
/*        where = "pipe-filter";
        smFile = "course1.txt";
        reportOnViolation = false;
        eventFilter = PipeFilterMessage.PIPE_IN_COURSE1;
        */ 
        probeEventFactory = new SienaProbeEventFactory ((SienaProbeBus )probeBus);
        doReporting = false;
        // Skipped the KXProbeHelper step as a hack, w/ assumption that all
        // probes have already been activated on system start
        //   (zensoul, May 03, 2002, HPDC paper deadline)
        // Then uncommented because gauge deletion stopped working properly for some reason
        protocolProbe = new KXProbeHelper (CONFIG_NAME, where, TARGET_CLASSES, this, probeBus);
        //probesDeployed();


	//initialize the protocol state machine
	    Global.debug("  Loading state machine'" + smFile + "'...");
            tracer = new SMTracer(smFile);
            tracer.dumpAlphabets();
            Global.debug("the state machine loaded!");                        
    }
    
    private void probesDeployed () {
        synchronized (probesActivated) {
            probesActivated = new Boolean (true);
        }
        
        IProbeEvent e = probeEventFactory.createProtocolEvent ("", CONFIG_NAME, where ); 
        probeBus.registerForEvent (e, this);
    }
    /** This method is used to configure the gauge.
     * @param configParams The parameters used to configure the gauge. 
      */       
    public boolean configure (StringPairVector configParams) {
    	
    	boolean validConfiguration = true;
        String value = configParams.getValue ("ReportOnViolation");
        if (value == null) validConfiguration = false;
          else
            this.reportOnViolation = Boolean.getBoolean(value);
        
        return validConfiguration;
  
    }
    
    public boolean consistentlyCreated () {
        boolean ret = false;
        synchronized (probesActivated) {
            ret = probesActivated.booleanValue ();
        }        
        
        return ret;
    }
       /** currently not supported
     */     
    public boolean queryAllValues (GaugeValueVector values) {
         /*synchronized (latency) {
            
            values.addElement ("latency", (String )mappings.get ("latency"), latency.toString ());
        }*/
        return false;       
    }
    
    public boolean queryState (StringPairVector setupParams, StringPairVector configParams, StringPairVector mappings) {
        this.setupParams.copyInto (setupParams);
        configParams.removeAllElements ();
        
        java.util.Enumeration enum = this.mappings.keys ();
        while (enum.hasMoreElements ()) {
            String key = (String )enum.nextElement ();
            mappings.addElement (key, (String )this.mappings.get (key));
        }
        
        return true;
    }
    
    public String queryValue (String valueName) {
        /*if (valueName.equals ("latency")) {
            String l = "";
            synchronized (latency) {
                l = latency.toString ();
            }
            return l;
        }
        else {
            return "";
        }*/
        return "";
    }
    
    public void reportNewValue () {
    	
    	return;
    	/*
        Double lLatency = null;
        synchronized (latency) {
            lLatency = latency;
        }
        gaugeBus.reportValue (gaugeID, "latency",
                              (String )mappings.get ("latency"),
                              lLatency.toString ());
        */
    }
    
    private boolean virgin = true;
    
    /** The callback invoked when a probe reports a single event.
     * @param event.  The event that was reported.
     */
    public void onProbeEvent (IProbeEvent event) {
    	
    	
        if (event instanceof SienaActivateResponseProbeEvent && virgin) {
            Global.debug ("Got activate response ... finishing creation");
            probesDeployed ();
            virgin = false;
        }
        else if (event instanceof SienaProtocolProbeEvent) {
        
            SienaProtocolProbeEvent protocolEvent =
                    (SienaProtocolProbeEvent )event;
            doReporting = true;            
            
            synchronized (lastEvent) {            	
              
              String action  = protocolEvent.getProbeEvent ().getAction ();              
              long time = ProbeMessageUtil.getTimestamp(action);
              action = ProbeMessageUtil.skipTimestamp(action);             
              Global.debug("action = " + action);           
            
              checkState(action);            
            }  
        }
    }
    
    
    private void checkState(String event) {
        
        boolean curState = true;
        String info = "default";     
           
        String componentName = ProbeMessageUtil.getComponentName(event);
        if (!componentName.equals(eventFilter)) return;
        event = ProbeMessageUtil.skipComponentName(event);        
        //System.out.println("event=" + event);
        lastEvent = (String)mapPG.get(event);
        event = componentName + "." + (String)mapPG.get(event);        
        
        //System.out.println("event2=" + event);

        errorType = 0;
        
        //System.out.println("reporting events111.......yooyoyoy...");
        
        if (! tracer.eventExists(event)) {  // no such event
           curState = false;           
           info = "'" + event + "' is not a valid event or command";                        
        }
          else 
             // trace the new event
                if (tracer.checkEBTrace(event)) { // event forms valid trace
                    info = tracer.printEventTrace();
                } else {  // event cause trace failure
                    info = tracer.printEventTrace();
                    info = info + "=> Last event '" + event + "' causes trace to fail.";
                    curState = false;
                }                
        
        GaugeValueVector values = new GaugeValueVector();
        values.addElement("violated", (String )mappings.get ("violated"), (new Boolean(!curState)).toString());
        values.addElement("LastEvent", (String )mappings.get ("LastEvent"), lastEvent);
        values.addElement("ErrorType", (String )mappings.get ("ErrorType"), (new Integer(errorType)).toString());
        values.addElement("EventHistory", (String )mappings.get ("EventHistory"), info);
        
                  Global.debug("reportOnViolation=" + reportOnViolation);
        
        if (!reportOnViolation || (reportOnViolation && !curState)) {
          Global.debug("reporting events.......yooyoyoy...");
          gaugeBus.reportMultipleValues(gaugeID, values);
        }        
    }

    
}
