/*
 * ServerLoadGaugeImpl.java
 *
 * Created on February 20, 2002, 2:48 PM
 */

package edu.cmu.cs.able.gauges.Siena.Demo;

import edu.cmu.cs.able.gaugeInfrastructure.util.*;
import edu.cmu.cs.able.gaugeInfrastructure.*;
import edu.cmu.cs.able.probebus.*;
import edu.cmu.cs.able.probebus.siena.*;
import edu.cmu.cs.able.probebus.events.*;
import edu.cmu.cs.able.probebus.siena.events.*;
/** Implementation of the ServerLoadGauge, conforming to gauge type:<br>
 * ServerLoadGauge = [<br>
 * &nbsp;&nbsp;type = edu.cmu.cs.able.gauges.Siena.Demo.ServerLoadGauge<br>
 * &nbsp;&nbsp;setup = [where : String]<br>
 * &nbsp;&nbsp;config = []<br>
 * &nbsp;&nbsp;values = [load : int]
 * @author Bradley Schmerl
 * @version 1.1
 * @changelog BRS 2/20/2002 Created after reorganization of gauge implementations
 * @changelog SWC 4/19/2002 Removed cumulative and averaging of load value.
 */
public class ServerLoadGaugeImpl extends edu.cmu.cs.able.gaugeInfrastructure.GaugeImpl implements IProbeConsumer {
    /** Where (the component or connector) the gauge is attached. (maps to the probe shell name)
     */    
    protected String where = null;
    protected String queue = null;
    private static final Double ZERO = new Double (0.0);
    private static final double ALPHA = 0.5;
    Double cumulative = ZERO;
    
    private Double lastReportedValue = ZERO;
    //int number = 0;  -zensoul, 4/19/2002
    int number = 1;
    public String gaugeType = "edu.cmu.cs.able.gauges.Siena.Demo.ServerLoadGauge";
    private static IProbeBus probeBus = null;
    private final String CONFIG_NAME = "ReqQueueConfig";
    private final String TARGET_CLASSES = "runtime.queue.ReqQueueClientThread";
    public KXProbeHelper loadProbe;
    private IProbeEventFactory probeEventFactory = null;    
    private Boolean probesActivated = new Boolean (false);
     /** Creates new ServerLoadGauge     
     * @param gaugeID The ID of the new gauge.
     * @param creatorID The ID of the gauge manager that created the gauge.
     * @param setupParams The parameters that the gauge uses to set itself up. The
     * valid setup parameters for this gauge are:<br>
     * &nbsp;&nbsp;where : String
     * @param mappings The property that the <pre>latency</pre> value is associated with.
     */
    public ServerLoadGaugeImpl (GaugeID gid, StringPairVector setup, StringPairVector mappings, GaugeReportingBus gBus, IProbeBus pBus) {
        super (gid, setup, mappings, gBus);
        reportingFrequency = 3000;
        this.probeBus = pBus;
        where = setup.getValue ("where");
        queue = setup.getValue ("queue");
        
        probeEventFactory = new SienaProbeEventFactory ((SienaProbeBus )probeBus);
        doReporting = false;
        Global.debug ("Creating KXProbeHelper, pbus = " + this.probeBus);
        // Skipped the KXProbeHelper step as a hack, w/ assumption that all
        // probes have already been activated on system start
        //   (zensoul, May 03, 2002, HPDC paper deadline)
        // Then uncommented because gauge deletion stopped working properly for some reason
        loadProbe = new KXProbeHelper (CONFIG_NAME, where, TARGET_CLASSES, this, this.probeBus);
//        probesDeployed();
    }
    
    private void probesDeployed () {
        Global.debug ("Probes have been deployed");
        synchronized (probesActivated) {
            probesActivated = new Boolean (true);
        }
//        finalizeCreation (myManagerID, true);
        IProbeEvent e = probeEventFactory.createBandwidthEvent ("", CONFIG_NAME, where ); 
        probeBus.registerForEvent (e, this);
    }
    
    public boolean configure (StringPairVector configParams) {
        return false;
    }
    
    public boolean consistentlyCreated () {
        boolean ret = false;
        synchronized (probesActivated) {
            ret = probesActivated.booleanValue ();
        }
        return ret;
    }
    
    public boolean queryAllValues (GaugeValueVector values) {
        synchronized (cumulative) {
            Double serverLoad = new Double (cumulative.doubleValue ()/number);
            values.addElement ("load", (String )mappings.get ("load"), serverLoad.toString ());
        }
        return true;
    }
    
    public boolean queryState (StringPairVector setupParams, StringPairVector configParams, StringPairVector mappings) {
        this.setupParams.copyInto (setupParams);
        configParams.removeAllElements ();
        
        java.util.Enumeration enum = this.mappings.keys ();
        while (enum.hasMoreElements ()) {
            String key = (String )enum.nextElement ();
            mappings.addElement (key, (String )this.mappings.get (key));
        }
        
        return true;
    }
    
    public String queryValue (String valueName) {
        if (valueName.equals ("load")) {
            String l = "";
            synchronized (cumulative) {
                Double serverLoad = new Double (cumulative.doubleValue ()/number);
                l = serverLoad.toString ();
            }
            return l;
        }
        else {
            return "";
        }
    }
    
    public void reportNewValue () {
        Double lLoad = ZERO;
        synchronized (cumulative) {
            lLoad = new Double (cumulative.doubleValue ()/number); //Double.toString (cumulative.doubleValue ()/number);
            //cumulative = ZERO;
            //number = 0;  -zensoul, 4/19/2002
        }
        if (!lLoad.equals (lastReportedValue)) {
            lastReportedValue = lLoad;
            gaugeBus.reportValue (gaugeID, "load",
                                (String )mappings.get ("load"),
                                lLoad.toString ());
        }
    }
    
    private boolean virgin = true;
    
    /** The callback invoked when a probe reports a single event.
     * @param e.  The event that was reported.
     */
    public void onProbeEvent (IProbeEvent event) {
        if (event instanceof SienaActivateResponseProbeEvent && virgin) {
            Global.debug ("Got activate response ... finishing creation");
            probesDeployed ();
            virgin = false;
        }
        else if (event instanceof SienaRequestQueueProbeEvent) {
            SienaRequestQueueProbeEvent loadEvent =
            (SienaRequestQueueProbeEvent )event;
            if (loadEvent.getProbeEvent ().getQueueNumber ().equals (queue)) {
                double newLoad = new Double (loadEvent.getProbeEvent ().getRequestQueueSize ()).doubleValue ();
                synchronized (cumulative) {
                    //number++;  -zensoul, 4/19/2002
                    //cumulative = new Double ((1.0-ALPHA)*cumulative.doubleValue () + ALPHA*newLoad);
                    cumulative = new Double(newLoad);
                }
                doReporting = true;
            }
        }
    }
    
}
