/*
 * SienaBandwidthGaugeMgr.java
 *
 * Created on November 14, 2001, 11:22 AM
 * $Header: D:/Projects/Repository/GaugeAPI/edu/cmu/cs/able/gauges/Siena/SienaBandwidthGaugeMgr.java,v 1.4 2002/05/24 18:13:31 schmerl Exp $
 * $Log: SienaBandwidthGaugeMgr.java,v $
 * Revision 1.4  2002/05/24 18:13:31  schmerl
 * Removed delegation to GaugeImpl in gauges - now done by SienaGauge
 * Removed full path to class form gauge type in gauge managers and gauges.
 * Gauge managers know where the class is to implement the gauge.
 *
 * Revision 1.3  2002/02/20 20:41:30  schmerl
 * 1. Fixed gauge deletion in the Siena version so that deletes don't timeout.
 * 2. Changed gauge implementation pattern so that gauge computation does not need to be included in both RMII and Siena versions.
 *
 * Revision 1.2  2002/01/31 16:39:11  schmerl
 * Added CVS header information
 *
 */

package edu.cmu.cs.able.gauges.Siena;
import java.util.*;
import java.io.*;
import edu.cmu.cs.able.gaugeInfrastructure.*;
import edu.cmu.cs.able.gaugeInfrastructure.util.*;
import edu.cmu.cs.able.gaugeInfrastructure.Siena.*;
import edu.cmu.cs.able.gaugeInfrastructure.Events.*;
/**
 *
 * @author  schmerl
 * @version 
 */
public class SienaBandwidthGaugeMgr extends edu.cmu.cs.able.gaugeInfrastructure.Siena.SienaGaugeMgr implements Runnable {

    private Hashtable p2out = new Hashtable (); // Process -> Thread
    private Hashtable out2p = new Hashtable (); // Thread -> Process
    private Hashtable threadState = new Hashtable ();  // Thread -> String
    public String senp = "";
    private String gaugeClassPackage = "edu.cmu.cs.able.gauges.Siena.SienaBandwidthGauge";
    
    private static boolean trackGaugeOutput = true;
    
    /** Creates new SienaBandwidthGaugeMgr
     * @param senp The Siena port to connect to to receive information
     * from gauge consumers and gauges.
     * @param register Indicates whether the gauge manager should be registed with the bus.
     */
    public SienaBandwidthGaugeMgr(String senp, boolean register) {
        super ();
        this.senp = senp;
        if (register) 
            Initialization.gaugeMgrBus.registerGaugeMgr (this);
    }

    /** Creates a new gauge as a separate process.
     * @param gauge The ID of the gauge to create.
     * @param setupParams The setup parameters to pass to the new gauge.
     * @param mappings The mappings to pass to the new gauge.
     * @return A "dummy" gauge control that has the new ID.
     */    
    public GaugeControl createGauge(GaugeID gauge, StringPairVector setupParams, StringPairVector mappings) {
   
        if (managesType (gauge.gaugeType)) {
            SienaGaugeMgrGaugeHandle gaugeHandle = new SienaGaugeMgrGaugeHandle (gauge);
            String args = " -mn " + gauge.modelName + " -mt " + gauge.modelType +
                          " -gt " + gauge.gaugeType + " -gn " + gauge.gaugeName +
                         " -creator \"" + getGaugeMgrID () + "\" -senp " + senp;
            try {
                // Start the gauge process
                System.out.println ("Creating " + gauge.gaugeType + args);
                Process gaugeProcess = Runtime.getRuntime ().exec ("java " + gaugeClassPackage + args);
                
                // Only start the output thread if gauges are being tracked
                if (trackGaugeOutput) {
                    Thread gaugeInOutput = new Thread (this, "IN" + gauge.toString ());
                    Thread gaugeErrOutput = new Thread (this, "ER" + gauge.toString ());
                    p2out.put (gaugeProcess.toString () + "IN", gaugeInOutput);
                    p2out.put (gaugeProcess.toString () + "ERR", gaugeErrOutput);
                    out2p.put (gaugeInOutput, gaugeProcess);
                    out2p.put (gaugeErrOutput, gaugeProcess);
                    gaugeInOutput.start ();
                    gaugeErrOutput.start ();
                }
                return gaugeHandle;
            }
            catch (IOException ex) {
                System.err.println ("Could not create gauge");
                ex.printStackTrace ();
                return null;
            }
        }
        return null;
    }
    
    /** Deletes the gauge process associated with the gauge ID.
     * @param gauge The gauge to be deleted.
     * @return Whether the process was destroyed successfully.
     */    
    public boolean deleteGauge(GaugeControl gauge) {
        Process gaugeProcess = (Process )gauges.get (gauge.getGaugeID ());
        if (gaugeProcess != null) {

            
            // Stop tracking gauge output
            if (trackGaugeOutput) {
                Thread output1Thread = (Thread )p2out.get (gaugeProcess.toString () + "IN");
                Thread output2Thread = (Thread )p2out.get (gaugeProcess.toString () + "ERR");
                // This may not make a gauge announce a delete event -- put it in the finalize of the gauge?
                threadState.put (output1Thread, "stop");
                threadState.put (output2Thread, "stop");
            }
            
            // Delete the gauge process
            try {
                gaugeProcess.destroy ();
            }
            catch (Throwable t) {
                System.err.println ("Could not destroy gauge");
                t.printStackTrace ();
 
                
                return false;
            }
        }
        DeletedEvent event = new DeletedEvent (gauge.getGaugeID ());
        event.gaugeMgrID = getGaugeMgrID ();
        event.status = true;
        Initialization.gaugeMgrBus.publishAnnouncement (event);
//        reportingBus.reportDeleted (event);
        return true;
    }
    
    /** Returns true if the gauge type is edu.cmu.cs.able.gauges.Siena.SienaBandwidthGauge
     * @param gaugeType
     * @return
     */    
    public boolean managesType(String gaugeType) {
        return gaugeType.equals ("RemosBandwidthGauge");
    }
    
    /** Returns the parameters that can be used to configure the gauge,
     * as well as the values reported by the gauge, for a particular
     * gauge type.
     * @param gaugeType The type of the gauge about which to get information.
     * @param configParamsMeta Will fill this vector with [("Frequency", "int")]
     * @param valuesMeta Will fill this vector with [("bandwidth", "float")]
     * @return
     */    
    public boolean queryMetaInfo(String gaugeType, StringPairVector configParamsMeta, StringPairVector valuesMeta) {
        if (managesType (gaugeType)) {
            configParamsMeta.addElement ("Frequency", "int");
            valuesMeta.addElement ("bandwidth", "float");
            return true;
        }
        else {
            return false;
        }
    }
    
    /** Implements the run method for threads. This method (if gauge stderr and stdin are being tracked)
     * will periodically output the values to stdout of the gauge manager.
     */    
    public void run () {
        
        Thread myThread = Thread.currentThread ();
        // Get the gauge process associated with this thread
        Process gaugeProcess = (Process )out2p.get (myThread);

        if (gaugeProcess != null) {
            BufferedReader in = null;
            // Create a new input channel depending on whether I'm the IN thread or the ERR thread
            if (myThread.getName ().substring (0,2).equals ("IN")) {
                in = new BufferedReader (new InputStreamReader (gaugeProcess.getInputStream ()));
            }
            else {
                in = new BufferedReader (new InputStreamReader (gaugeProcess.getErrorStream ()));
            }
           
            // if I'm still running, output any info I get from the gauge process
            while (threadState.get (myThread) == null) { 
                try {
                    String n = in.readLine ();
                    while (n != null) {
                        System.out.println (myThread.getName () + "==>" + n);
                        n = in.readLine ();
                    }
                }
                catch (IOException e) {
                    
                }
                catch (Exception e) {}
                
                // Sleep for a second 
                try {
                    Thread.sleep (1000);
                }
                catch (Exception e) {
                }
            }
            
        }
    }
    
    /** A way to start the gauge manager from the command line.
     * @param args -senp <i>siena port</i><br>
     * [-debug] [-silent]<br>
     *
     * -debug indicates whether debugging information should be displayed<br>
     * -silent indicates whether the gauge outputs should be tracked.
     *
     */    
    public static void main(String[] args) {
        
        boolean setup = false;
        for (int i = 0; i < args.length; i++) {
            
            if (args [i].equals ("-senp")) {
                edu.cmu.cs.able.gaugeInfrastructure.Siena.Initialization.initSiena (args [++i], Initialization.GAUGEMGR);
                setup = true;
            } 
            else if (args [i].equals ("-debug")) {
                edu.cmu.cs.able.gaugeInfrastructure.util.Global.debugFlag = true;
            }
            else if (args [i].equals ("-silent")) {
                trackGaugeOutput = false;
            }
            else {
                printUsageMessage ();
                System.exit (0);
            }
        }
            
        if (!setup) {
            printUsageMessage ();
            System.exit (0);
        }
        
        SienaBandwidthGaugeMgr gm = new SienaBandwidthGaugeMgr (args [1], true);
        
        while (true) {
            try {
                Thread.sleep (100000000);
            }
            catch (Exception e) {
            }
        }
    }
    
    /** Prints the usage message.
     */    
    public static void printUsageMessage () {
        System.err.println ("Usage: SienaBandwidthGaugeMgr -senp <siena-port> [-debug] [-silent]");
        
    }
  
    
}

